using System;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Runtime.InteropServices;
using System.Configuration;
using Microsoft.ApplicationBlocks.ConfigurationManagement;

namespace CustomConfigurationSectionHandler
{
	/// <summary>
	/// Summary description for ProxySectionHandler.
	/// </summary>
	[ComVisible(false)]
	public class ProxySectionHandler : IConfigurationSectionHandler, IConfigurationSectionHandlerWriter
	{
		#region Declarations
		private XmlSerializer _xs;
		#endregion

		#region Constructors
		public ProxySectionHandler()
		{
			// Initializes a new instance of the XmlSerializer class that can serialize objects
			// of the specified type into XML documents, and vice versa
			_xs = new XmlSerializer(typeof(ProxyConfigurationData));
		}
		#endregion

		#region IConfigurationSectionHandler Implementation
		public object Create(object parent, object hmm, XmlNode configSection)
		{
			ProxyConfigurationData cconfigData = null;
			// Serializes the specified objet and writes the XML document to a file using the specified Stream
			cconfigData = _xs.Deserialize(new StringReader(configSection.OuterXml)) as ProxyConfigurationData;

			return cconfigData;
		}
		#endregion

		#region IConfigurationSectionHandlerWriter Implementation
		public XmlNode Serialize(object value)
		{
			try
			{
				StringWriter sw = new StringWriter(System.Globalization.CultureInfo.CurrentUICulture);
				// Serializes the specified object and writes the XML document to a file using the specified Stream
				_xs.Serialize(sw, value);
				
				XmlDocument doc = new XmlDocument();
				doc.LoadXml(sw.ToString());

				return doc.ChildNodes[1];
			}
			catch(Exception e)
			{
				throw new ConfigurationException("The section handler can't be serialized.", e);
			}
		}
		#endregion
	}

	/*
	 * The following listing are the custom ReadOnlyProxySection handler.
	 * 
	[ComVisible(false)]
	public class ReadOnlyProxySectionHandler : IConfigurationSectionHandler
	{
		#region Declarations
		private XmlSerializer _xs;
		#endregion

		#region Constructors
		public ReadOnlyProxySectionHandler()
		{
			_xs = new XmlSerializer( typeof ( ProxyConfigurationData ) );
		}
		#endregion

		#region IConfigurationSectionHandler Implementation
		public object Create( object parent, object hmm, XmlNode configSection )
		{ 
			ProxyConfigurationData cconfigData = null;

			cconfigData = _xs.Deserialize( new StringReader( configSection.OuterXml ) ) as ProxyConfigurationData;

			return cconfigData;
		}
		#endregion
	}
	*/
}
